import { auth } from "@/auth";
import * as React from "react";
import {
  Sidebar,
  SidebarContent,
  SidebarGroup,
  SidebarGroupContent,
  SidebarGroupLabel,
  SidebarMenu,
  SidebarMenuButton,
  SidebarMenuItem,
  SidebarMenuSub,
  SidebarRail,
} from "@/components/ui/sidebar";
import { GitEntry } from "gitea-js";
import {
  Collapsible,
  CollapsibleContent,
  CollapsibleTrigger,
} from "@/components/ui/collapsible";
import { retrieveTreeStructure } from "@/app/actions";
import { ChevronRight, File, Folder } from "lucide-react";

interface FileTree {
  name: string;
  type: "blob" | "tree";
  children?: { [key: string]: FileTree };
}

export function buildFileTree(tree: GitEntry[]): FileTree {
  const root: FileTree = { name: "", type: "tree", children: {} };

  tree.forEach((entry: GitEntry) => {
    if (!entry.path) return;

    const pathParts = entry.path.split("/");
    let currentLevel = root;

    pathParts.forEach((part: string) => {
      if (!currentLevel.children) {
        currentLevel.children = {};
      }

      if (!currentLevel.children[part]) {
        currentLevel.children[part] = {
          name: part,
          type: entry.type as "blob" | "tree",
          children: {},
        };
      }

      currentLevel = currentLevel.children[part];
    });
  });

  if (root.children) {
    const sortedChildren = Object.values(root.children).sort((a, b) => {
      if (a.type === "tree" && b.type === "blob") return -1;
      if (a.type === "blob" && b.type === "tree") return 1;
      return a.name.localeCompare(b.name);
    });

    root.children = sortedChildren.reduce((acc, child) => {
      acc[child.name] = child;
      return acc;
    }, {} as { [key: string]: FileTree });
  }

  return root;
}

export async function PlaygroundSidebar({
  ...props
}: React.ComponentProps<typeof Sidebar>) {
  const session = await auth();
  const username = session?.user?.name ?? "";
  let fileTree: FileTree = { name: "", type: "tree", children: {} };
  if (username) {
    const tree: GitEntry[] = await retrieveTreeStructure(
      username,
      "playground",
      "main"
    );
    fileTree = buildFileTree(tree);
  }

  return (
    <Sidebar {...props}>
      <SidebarContent>
        <SidebarGroup>
          <SidebarGroupLabel>Files</SidebarGroupLabel>
          <SidebarGroupContent>
            <SidebarMenu>
              {fileTree.children &&
                Object.values(fileTree.children).map((item, index) => (
                  <Tree key={index} item={item} />
                ))}
            </SidebarMenu>
          </SidebarGroupContent>
        </SidebarGroup>
      </SidebarContent>
      <SidebarRail />
    </Sidebar>
  );
}

function Tree({ item }: { item: FileTree }) {
  const { name, type, children } = item;
  if (type === "blob") {
    return (
      <SidebarMenuButton className="data-[active=true]:bg-transparent">
        <File />
        {name}
      </SidebarMenuButton>
    );
  }

  return (
    <SidebarMenuItem>
      <Collapsible className="group/collapsible [&[data-state=open]>button>svg:first-child]:rotate-90">
        <CollapsibleTrigger asChild>
          <SidebarMenuButton>
            <ChevronRight className="transition-transform" />
            <Folder />
            {name}
          </SidebarMenuButton>
        </CollapsibleTrigger>
        <CollapsibleContent>
          <SidebarMenuSub>
            {children &&
              Object.values(children).map((subItem, index) => (
                <Tree key={index} item={subItem} />
              ))}
          </SidebarMenuSub>
        </CollapsibleContent>
      </Collapsible>
    </SidebarMenuItem>
  );
}
